#[derive(PartialEq, Debug)]
pub struct Message<'a> {
    pub command: &'a str,
    pub args: Vec<&'a str>,
}

pub type Result<T> = std::result::Result<T, Error>;

#[repr(u8)]
#[derive(PartialEq, Debug)]
pub enum Error {
    BadParse,
    UnknownCommand,
}

impl<'a> Message<'a> {
    pub fn parse(text: &'a str) -> Result<Message<'a>> {
	let re = regex::Regex::new(r"^([A-Z_]+):\s*(.*)").unwrap();
	match re.captures(text) {
            Some(captures) => {
		if captures.len() < 3 {
		    Err(Error::BadParse)
		} else {
		    let command = captures.get(1).unwrap().as_str();
		    let args = captures.get(2).unwrap().as_str()
			.split(',')
			.map(|s| s.trim())
			.collect();
                    Ok(Message { command, args })
                }
            }
	    None => Err(Error::BadParse),
        }
    }
}

#[cfg(test)]
mod test {
    use super::*;
    #[test]
    fn test_parse() -> Result<()> {
	let text = "COMMAND: arg1, arg2";
	let msg = Message::parse(text)?;
	assert_eq!(Message {
	    command: "COMMAND",
	    args: vec!["arg1", "arg2"],
	}, msg);

	Ok(())
    }
}
